@@ -1,9 +1,7 @@
-# -*- coding: utf-8 -*-
-
 """
 The MIT License (MIT)
 
-Copyright (c) 2015-2020 Rapptz
+Copyright (c) 2015-present Rapptz
 
 Permission is hereby granted, free of charge, to any person obtaining a
 copy of this software and associated documentation files (the "Software"),
@@ -57,6 +55,11 @@ try:
 except ImportError:
     has_nacl = False
 
+__all__ = (
+    'VoiceProtocol',
+    'VoiceClient',
+)
+
 log = logging.getLogger(__name__)
 
 class VoiceProtocol:
@@ -70,7 +73,7 @@ class VoiceProtocol:
 
     These classes are passed to :meth:`abc.Connectable.connect`.
 
-    .. _Lavalink: https://github.com/Frederikam/Lavalink
+    .. _Lavalink: https://github.com/freyacodes/Lavalink
 
     Parameters
     ------------
@@ -220,6 +223,7 @@ class VoiceClient(VoiceProtocol):
         self._player = None
         self.encoder = None
         self._lite_nonce = 0
+        self.ws = None
 
     warn_nacl = not has_nacl
     supported_modes = (
@@ -364,6 +368,8 @@ class VoiceClient(VoiceProtocol):
             self._runner = self.loop.create_task(self.poll_voice_ws(reconnect))
 
     async def potential_reconnect(self):
+        # Attempt to stop the player thread from playing early
+        self._connected.clear()
         self.prepare_handshake()
         self._potentially_reconnecting = True
         try:
@@ -555,7 +561,7 @@ class VoiceClient(VoiceProtocol):
             raise ClientException('Already playing audio.')
 
         if not isinstance(source, AudioSource):
-            raise TypeError('source must an AudioSource not {0.__class__.__name__}'.format(source))
+            raise TypeError(f'source must an AudioSource not {source.__class__.__name__}')
 
         if not self.encoder and not source.is_opus():
             self.encoder = opus.Encoder()
@@ -598,7 +604,7 @@ class VoiceClient(VoiceProtocol):
     @source.setter
     def source(self, value):
         if not isinstance(value, AudioSource):
-            raise TypeError('expected AudioSource not {0.__class__.__name__}.'.format(value))
+            raise TypeError(f'expected AudioSource not {value.__class__.__name__}.')
 
         if self._player is None:
             raise ValueError('Not playing anything.')
